//  Copyright (C) 2021, Christoph Peters, Karlsruhe Institute of Technology
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.


// This file is automatically generated. It implements GLM-like functionality
// but is less complete. The benefit is that it is a painless dependency.
#pragma once

struct vec2 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y; };
	};
};

struct vec3 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y, z; };
		struct { Float x_1; vec2 yz; };
		struct { vec2 xy; Float z_1; };
	};
};

struct vec4 {
	inline Float& operator[](unsigned int entry_index) {
		return *((&x) + entry_index);
	}
	inline Float operator[](unsigned int entry_index) const {
		return *((&x) + entry_index);
	}

	union {
		struct { Float x, y, z, w; };
		struct { Float x_1; vec3 yzw; };
		struct { vec2 xy; vec2 zw; };
		struct { vec3 xyz; Float w_1; };
		struct { Float x_3; vec2 yz; Float w_2; };
	};
};

struct mat2 {
	inline mat2() {}
	inline mat2(Float diagonal) {
		columns[0][0] = diagonal;
		columns[0][1] = 0;
		columns[1][0] = 0;
		columns[1][1] = diagonal;
	}
	inline mat2(Float col_0_row_0, Float col_0_row_1, Float col_1_row_0, Float col_1_row_1) {
		columns[0][0] = col_0_row_0;
		columns[0][1] = col_0_row_1;
		columns[1][0] = col_1_row_0;
		columns[1][1] = col_1_row_1;
	}
	inline mat2(vec2 col_0, vec2 col_1) {
		columns[0] = col_0;
		columns[1] = col_1;
	}
	inline vec2& operator[](unsigned int col) {
		return columns[col];
	}
	inline const vec2& operator[](unsigned int col) const{
		return columns[col];
	}

	vec2 columns[2];
};

inline vec2 make_vec2(Float scalar) {
	vec2 result;
	result.x = scalar;
	result.y = scalar;
	return result;
}
inline vec2 make_vec2(Float x, Float y) {
	vec2 result;
	result.x = x;
	result.y = y;
	return result;
}

inline vec2 operator-(const vec2& rhs) {
	vec2 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	return result;
}

inline vec2 operator+(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	return result;
}

inline vec2 operator+(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	return result;
}

inline vec2 operator+(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	return result;
}

inline const vec2& operator+=(vec2& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	return lhs;
}

inline const vec2& operator+=(vec2& lhs, const vec2& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	return lhs;
}

inline vec2 operator-(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	return result;
}

inline vec2 operator-(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	return result;
}

inline vec2 operator-(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	return result;
}

inline const vec2& operator-=(vec2& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	return lhs;
}

inline const vec2& operator-=(vec2& lhs, const vec2& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	return lhs;
}

inline vec2 operator*(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	return result;
}

inline vec2 operator*(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	return result;
}

inline vec2 operator*(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	return result;
}

inline const vec2& operator*=(vec2& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	return lhs;
}

inline const vec2& operator*=(vec2& lhs, const vec2& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	return lhs;
}

inline vec2 operator/(Float lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	return result;
}

inline vec2 operator/(const vec2& lhs, Float rhs) {
	vec2 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	return result;
}

inline vec2 operator/(const vec2& lhs, const vec2& rhs) {
	vec2 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	return result;
}

inline const vec2& operator/=(vec2& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	return lhs;
}

inline const vec2& operator/=(vec2& lhs, const vec2& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	return lhs;
}

inline vec3 make_vec3(Float scalar) {
	vec3 result;
	result.x = scalar;
	result.y = scalar;
	result.z = scalar;
	return result;
}
inline vec3 make_vec3(Float x, vec2 yz) {
	vec3 result;
	result.x = x;
	result.yz = yz;
	return result;
}
inline vec3 make_vec3(vec2 xy, Float z) {
	vec3 result;
	result.xy = xy;
	result.z = z;
	return result;
}
inline vec3 make_vec3(Float x, Float y, Float z) {
	vec3 result;
	result.x = x;
	result.y = y;
	result.z = z;
	return result;
}

inline vec3 operator-(const vec3& rhs) {
	vec3 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	result.z = -rhs.z;
	return result;
}

inline vec3 operator+(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	result.z = lhs + rhs.z;
	return result;
}

inline vec3 operator+(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	result.z = lhs.z + rhs;
	return result;
}

inline vec3 operator+(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	result.z = lhs.z + rhs.z;
	return result;
}

inline const vec3& operator+=(vec3& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	lhs.z += rhs;
	return lhs;
}

inline const vec3& operator+=(vec3& lhs, const vec3& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	lhs.z += rhs.z;
	return lhs;
}

inline vec3 operator-(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	result.z = lhs - rhs.z;
	return result;
}

inline vec3 operator-(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	result.z = lhs.z - rhs;
	return result;
}

inline vec3 operator-(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	result.z = lhs.z - rhs.z;
	return result;
}

inline const vec3& operator-=(vec3& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	lhs.z -= rhs;
	return lhs;
}

inline const vec3& operator-=(vec3& lhs, const vec3& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	lhs.z -= rhs.z;
	return lhs;
}

inline vec3 operator*(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	result.z = lhs * rhs.z;
	return result;
}

inline vec3 operator*(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	result.z = lhs.z * rhs;
	return result;
}

inline vec3 operator*(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	result.z = lhs.z * rhs.z;
	return result;
}

inline const vec3& operator*=(vec3& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	lhs.z *= rhs;
	return lhs;
}

inline const vec3& operator*=(vec3& lhs, const vec3& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	lhs.z *= rhs.z;
	return lhs;
}

inline vec3 operator/(Float lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	result.z = lhs / rhs.z;
	return result;
}

inline vec3 operator/(const vec3& lhs, Float rhs) {
	vec3 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	result.z = lhs.z / rhs;
	return result;
}

inline vec3 operator/(const vec3& lhs, const vec3& rhs) {
	vec3 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	result.z = lhs.z / rhs.z;
	return result;
}

inline const vec3& operator/=(vec3& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	lhs.z /= rhs;
	return lhs;
}

inline const vec3& operator/=(vec3& lhs, const vec3& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	lhs.z /= rhs.z;
	return lhs;
}

inline vec4 make_vec4(Float scalar) {
	vec4 result;
	result.x = scalar;
	result.y = scalar;
	result.z = scalar;
	result.w = scalar;
	return result;
}
inline vec4 make_vec4(Float x, vec3 yzw) {
	vec4 result;
	result.x = x;
	result.yzw = yzw;
	return result;
}
inline vec4 make_vec4(vec2 xy, vec2 zw) {
	vec4 result;
	result.xy = xy;
	result.zw = zw;
	return result;
}
inline vec4 make_vec4(vec3 xyz, Float w) {
	vec4 result;
	result.xyz = xyz;
	result.w = w;
	return result;
}
inline vec4 make_vec4(Float x, Float y, vec2 zw) {
	vec4 result;
	result.x = x;
	result.y = y;
	result.zw = zw;
	return result;
}
inline vec4 make_vec4(Float x, vec2 yz, Float w) {
	vec4 result;
	result.x = x;
	result.yz = yz;
	result.w = w;
	return result;
}
inline vec4 make_vec4(vec2 xy, Float z, Float w) {
	vec4 result;
	result.xy = xy;
	result.z = z;
	result.w = w;
	return result;
}
inline vec4 make_vec4(Float x, Float y, Float z, Float w) {
	vec4 result;
	result.x = x;
	result.y = y;
	result.z = z;
	result.w = w;
	return result;
}

inline vec4 operator-(const vec4& rhs) {
	vec4 result;
	result.x = -rhs.x;
	result.y = -rhs.y;
	result.z = -rhs.z;
	result.w = -rhs.w;
	return result;
}

inline vec4 operator+(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs + rhs.x;
	result.y = lhs + rhs.y;
	result.z = lhs + rhs.z;
	result.w = lhs + rhs.w;
	return result;
}

inline vec4 operator+(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x + rhs;
	result.y = lhs.y + rhs;
	result.z = lhs.z + rhs;
	result.w = lhs.w + rhs;
	return result;
}

inline vec4 operator+(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x + rhs.x;
	result.y = lhs.y + rhs.y;
	result.z = lhs.z + rhs.z;
	result.w = lhs.w + rhs.w;
	return result;
}

inline const vec4& operator+=(vec4& lhs, Float rhs) {
	lhs.x += rhs;
	lhs.y += rhs;
	lhs.z += rhs;
	lhs.w += rhs;
	return lhs;
}

inline const vec4& operator+=(vec4& lhs, const vec4& rhs) {
	lhs.x += rhs.x;
	lhs.y += rhs.y;
	lhs.z += rhs.z;
	lhs.w += rhs.w;
	return lhs;
}

inline vec4 operator-(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs - rhs.x;
	result.y = lhs - rhs.y;
	result.z = lhs - rhs.z;
	result.w = lhs - rhs.w;
	return result;
}

inline vec4 operator-(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x - rhs;
	result.y = lhs.y - rhs;
	result.z = lhs.z - rhs;
	result.w = lhs.w - rhs;
	return result;
}

inline vec4 operator-(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x - rhs.x;
	result.y = lhs.y - rhs.y;
	result.z = lhs.z - rhs.z;
	result.w = lhs.w - rhs.w;
	return result;
}

inline const vec4& operator-=(vec4& lhs, Float rhs) {
	lhs.x -= rhs;
	lhs.y -= rhs;
	lhs.z -= rhs;
	lhs.w -= rhs;
	return lhs;
}

inline const vec4& operator-=(vec4& lhs, const vec4& rhs) {
	lhs.x -= rhs.x;
	lhs.y -= rhs.y;
	lhs.z -= rhs.z;
	lhs.w -= rhs.w;
	return lhs;
}

inline vec4 operator*(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs * rhs.x;
	result.y = lhs * rhs.y;
	result.z = lhs * rhs.z;
	result.w = lhs * rhs.w;
	return result;
}

inline vec4 operator*(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x * rhs;
	result.y = lhs.y * rhs;
	result.z = lhs.z * rhs;
	result.w = lhs.w * rhs;
	return result;
}

inline vec4 operator*(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x * rhs.x;
	result.y = lhs.y * rhs.y;
	result.z = lhs.z * rhs.z;
	result.w = lhs.w * rhs.w;
	return result;
}

inline const vec4& operator*=(vec4& lhs, Float rhs) {
	lhs.x *= rhs;
	lhs.y *= rhs;
	lhs.z *= rhs;
	lhs.w *= rhs;
	return lhs;
}

inline const vec4& operator*=(vec4& lhs, const vec4& rhs) {
	lhs.x *= rhs.x;
	lhs.y *= rhs.y;
	lhs.z *= rhs.z;
	lhs.w *= rhs.w;
	return lhs;
}

inline vec4 operator/(Float lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs / rhs.x;
	result.y = lhs / rhs.y;
	result.z = lhs / rhs.z;
	result.w = lhs / rhs.w;
	return result;
}

inline vec4 operator/(const vec4& lhs, Float rhs) {
	vec4 result;
	result.x = lhs.x / rhs;
	result.y = lhs.y / rhs;
	result.z = lhs.z / rhs;
	result.w = lhs.w / rhs;
	return result;
}

inline vec4 operator/(const vec4& lhs, const vec4& rhs) {
	vec4 result;
	result.x = lhs.x / rhs.x;
	result.y = lhs.y / rhs.y;
	result.z = lhs.z / rhs.z;
	result.w = lhs.w / rhs.w;
	return result;
}

inline const vec4& operator/=(vec4& lhs, Float rhs) {
	lhs.x /= rhs;
	lhs.y /= rhs;
	lhs.z /= rhs;
	lhs.w /= rhs;
	return lhs;
}

inline const vec4& operator/=(vec4& lhs, const vec4& rhs) {
	lhs.x /= rhs.x;
	lhs.y /= rhs.y;
	lhs.z /= rhs.z;
	lhs.w /= rhs.w;
	return lhs;
}

inline mat2 operator-(const mat2& rhs) {
	mat2 result;
	result.columns[0] = -rhs[0];
	result.columns[1] = -rhs[1];
	return result;
}

inline mat2 operator+(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs + rhs.columns[0];
	result.columns[1] = lhs + rhs.columns[1];
	return result;
}

inline mat2 operator+(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] + rhs;
	result.columns[1] = lhs.columns[1] + rhs;
	return result;
}

inline mat2 operator+(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] + rhs.columns[0];
	result.columns[1] = lhs.columns[1] + rhs.columns[1];
	return result;
}

inline const mat2& operator+=(mat2& lhs, Float rhs) {
	lhs.columns[0] += rhs;
	lhs.columns[1] += rhs;
	return lhs;
}

inline const mat2& operator+=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] += rhs.columns[0];
	lhs.columns[1] += rhs.columns[1];
	return lhs;
}

inline mat2 operator-(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs - rhs.columns[0];
	result.columns[1] = lhs - rhs.columns[1];
	return result;
}

inline mat2 operator-(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] - rhs;
	result.columns[1] = lhs.columns[1] - rhs;
	return result;
}

inline mat2 operator-(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] - rhs.columns[0];
	result.columns[1] = lhs.columns[1] - rhs.columns[1];
	return result;
}

inline const mat2& operator-=(mat2& lhs, Float rhs) {
	lhs.columns[0] -= rhs;
	lhs.columns[1] -= rhs;
	return lhs;
}

inline const mat2& operator-=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] -= rhs.columns[0];
	lhs.columns[1] -= rhs.columns[1];
	return lhs;
}

inline mat2 operator*(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs * rhs.columns[0];
	result.columns[1] = lhs * rhs.columns[1];
	return result;
}

inline mat2 operator*(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] * rhs;
	result.columns[1] = lhs.columns[1] * rhs;
	return result;
}

inline mat2 operator*(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] * rhs.columns[0];
	result.columns[1] = lhs.columns[1] * rhs.columns[1];
	return result;
}

inline const mat2& operator*=(mat2& lhs, Float rhs) {
	lhs.columns[0] *= rhs;
	lhs.columns[1] *= rhs;
	return lhs;
}

inline const mat2& operator*=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] *= rhs.columns[0];
	lhs.columns[1] *= rhs.columns[1];
	return lhs;
}

inline mat2 operator/(Float lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs / rhs.columns[0];
	result.columns[1] = lhs / rhs.columns[1];
	return result;
}

inline mat2 operator/(const mat2& lhs, Float rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] / rhs;
	result.columns[1] = lhs.columns[1] / rhs;
	return result;
}

inline mat2 operator/(const mat2& lhs, const mat2& rhs) {
	mat2 result;
	result.columns[0] = lhs.columns[0] / rhs.columns[0];
	result.columns[1] = lhs.columns[1] / rhs.columns[1];
	return result;
}

inline const mat2& operator/=(mat2& lhs, Float rhs) {
	lhs.columns[0] /= rhs;
	lhs.columns[1] /= rhs;
	return lhs;
}

inline const mat2& operator/=(mat2& lhs, const mat2& rhs) {
	lhs.columns[0] /= rhs.columns[0];
	lhs.columns[1] /= rhs.columns[1];
	return lhs;
}

inline vec2 operator*(const mat2& lhs, const vec2& rhs) {
	vec2 result;
	result[0] = lhs[0][0] * rhs[0] + lhs[1][0] * rhs[1];
	result[1] = lhs[0][1] * rhs[0] + lhs[1][1] * rhs[1];
	return result;
}

inline mat2 transpose(const mat2& rhs) {
	mat2 result;
	result.columns[0][0] = rhs.columns[0][0];
	result.columns[1][0] = rhs.columns[0][1];
	result.columns[0][1] = rhs.columns[1][0];
	result.columns[1][1] = rhs.columns[1][1];
	return result;
}

