#  Copyright (C) 2021, Christoph Peters, Karlsruhe Institute of Technology
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.


from material_conversion import complete_materials, convert_materials


def complete_living_room_materials():
    """
    Creates missing texture files for the living room scene.
    """
    materials = dict(
        apple=(None, 0.4),
        black_raught_iron=(0.016, (1.0, 0.7, 1.0)),
        blind_material=((0.8, 0.772, 0.717), 1.0),
        book_spines=(None, 0.1),
        candle_holders=((0.055, 0.027, 0.02), 0.1),
        carpet=(None, 1.0),
        chrome_dull=(0.8, 0.173),
        cushion_purple_yellow_stripe=(None, 0.9),
        cushion_stripe_red=(None, 0.9),
        cushion_stripe_green_circles=(None, 0.9),
        cushion_stripe_purple=(None, 0.9),
        glossy_white_wood=(0.906, 0.045),
        lampshade_inner=(0.970, 0.8),
        lamp_upright_base=(0.855, 0.071),
        magazine=(None, 0.032),
        marble_harf=(0.02, 0.283),
        matte_black_wood=(0.02, 0.224),
        matte_white_wood=(0.906, 0.224),
        plain_wood=((0.267, 0.217, 0.174), 0.4),
        rad_panels_enamel=(0.906, 0.447),
        rad_knob_outer=((0.1, 0.03, 0.01), (1.0, 0.447, 1.0)),
        rad_knob_center=(0.157, (1.0, 0.447, 1.0)),
        radio_dial_back=((0.208, 0.016, 0.01), 1.0),
        radio_knob=((0.448, 0.376, 0.161), 1.0),
        radio_plastic=((0.799, 0.753, 0.546), 0.447),
        shade_paper=(None, 1.0),
        shade_stripes_outer=(None, 0.8),
        steel_handle=(0.8, (1.0, 0.447, 1.0)),
        tv_bevel=(0.005, 0.032),
        tv_screen=(0.033, 0.089),
        wall=((0.736, 0.724, 0.716), 0.8),
        white_leather=(None, 0.361),
        white_plastic=(0.8, 0.447),
        white_porcelain=(0.8, 0.1),
    )
    complete_materials("../Models/living_room/textures", materials)


def complete_attic_materials():
    """
    Creates missing texture files for the attic scene.
    """
    materials = dict(
        bed_frame=(0.9, 0.8),
        bed_sheet=(0.8, 0.75),
        bed_wood=((0.486897, 0.325651, 0.153488), 0.3),
        black_plastic=(0.02, 0.3),
        brass=((0.401969, 0.159576, 0.007421), (1.0, 0.5, 1.0)),
        gold=((0.8, 0.694896, 0.306), (1.0, 0.376, 1.0)),
        lamp_shade=(0.8, 0.9),
        mattress=((0.577581, 0.590619, 0.610496), 1.0),
        pillow=(0.8, 0.8),
        sofa=((1.000000, 0.378317, 0.098382), 0.4),
        sofaseat=((0.595531, 0.229704, 0.122099), 0.568),
        sofastand=((0.312841, 0.312841, 0.312841), 0.273),
        wood=(0.8, 0.35),
        woven_cloth=(0.8, 0.9),
    )
    complete_materials("../Models/attic/textures", materials)


def complete_mis_plane_materials():
    """
    Creates missing texture files for the plane model that is used to
    illustrate pros and cons of different MIS variants.
    """
    materials = dict(
        occluder=((0.3, 0.3, 0.8), 1.0),
        diffuse=((0.05, 0.52, 0.9), 1.0),
        glossy=(0.8, 0.1),
    )
    complete_materials("../Models/mis_plane/textures", materials)


def complete_roughness_plane_materials():
    """
    Creates missing texture files for the planes with different roughness.
    """
    materials = dict(
        high_roughness=(0.8, 0.3),
        medium_roughness=(0.8, 0.2),
        low_roughness=(0.8, 0.0),
    )
    complete_materials("../Models/roughness_planes/textures", materials)


if __name__ == "__main__":
    # convert_materials("../vulkan_renderer/data/Arcade_textures", "../Models/Arcade/Textures", False)
    # complete_materials("../Models/Bistro_v4/textures")
    # convert_materials("../vulkan_renderer/data/Bistro_textures", "../Models/Bistro_v4/textures")
    # complete_materials("../Models/cornell_box/textures")
    # convert_materials("../vulkan_renderer/data/cornell_box_textures", "../Models/cornell_box/textures")
    # complete_materials("../Models/shadowed_plane/textures")
    # convert_materials("../vulkan_renderer/data/shadowed_plane_textures", "../Models/shadowed_plane/textures")
    # convert_materials("../vulkan_renderer/data/poltergeist_textures", "../Models/poltergeist/textures")
    # complete_living_room_materials()
    # convert_materials("../vulkan_renderer/data/living_room_textures", "../Models/living_room/textures")
    # complete_attic_materials()
    # convert_materials("../vulkan_renderer/data/attic_textures", "../Models/attic/textures")
    # complete_mis_plane_materials()
    # convert_materials("../vulkan_renderer/data/mis_plane_textures", "../Models/mis_plane/textures")
    complete_roughness_plane_materials()
    convert_materials("../vulkan_renderer/data/roughness_planes_textures", "../Models/roughness_planes/textures")
    print("Done.")
